/*
	pcmwav.c - source file for PCM WAV I/O - v0.241
	(c) 2000, 2001 Manuel Kasper <mk@neon1.net>

	This file is part of normalize.

	normalize is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	normalize is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "pcmwav.h"
#include <stdio.h>
#include <windows.h>

char pcmwav_error[256];

int pcmwav_open(char *fname, DWORD access, pcmwavfile *opwf) {

	RIFFhdr		rhdr;
	fmt_sub		fmt;
	data_sub	data;
	DWORD		nread, sig;

	opwf->winfile = CreateFile(fname, access, 0, NULL,
						OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if (opwf->winfile == INVALID_HANDLE_VALUE) {
		sprintf(pcmwav_error, "Cannot open file %s.\n", fname);
		return 0;
	}

	// Read RIFF header
	ReadFile(opwf->winfile, &rhdr, sizeof(rhdr), &nread, NULL);

	// Check it
	if ((rhdr.ChunkID != 0x46464952) || (rhdr.Format != 0x45564157)) {
		sprintf(pcmwav_error, "This is not a PCM WAV file.\n");
		CloseHandle(opwf->winfile);
		return 0;
	}

	// Read subchunk 1
	ReadFile(opwf->winfile, &fmt, sizeof(fmt), &nread, NULL);

	// Check it
	if ((fmt.Subchunk1ID != 0x20746D66) || (fmt.AudioFormat != 1)) {
		sprintf(pcmwav_error, "Error in subchunk 1: this is not a PCM WAV file.\n");
		CloseHandle(opwf->winfile);
		return 0;
	}

	opwf->bitspersample = fmt.BitsPerSample;

	if ((opwf->bitspersample != 8) && (opwf->bitspersample != 16)) {
		sprintf(pcmwav_error, "Can only deal with 8-bit or 16-bit samples.\n");
		return 0;
	}

	// Skip any extra header bytes
	if (fmt.Subchunk1Size - 16) {
		SetFilePointer(opwf->winfile, fmt.Subchunk1Size - 16, NULL, FILE_CURRENT);
	}
	
	// See if there's a 'fact' subchunk (some recorders insert this)
	ReadFile(opwf->winfile, &sig, sizeof(sig), &nread, NULL);
	
	if (sig == 0x74636166) {
		// Read length
		ReadFile(opwf->winfile, &sig, sizeof(sig), &nread, NULL);
		if (sig)
			SetFilePointer(opwf->winfile, sig, NULL, FILE_CURRENT);
		
		// Read subchunk 2
		ReadFile(opwf->winfile, &data, sizeof(data), &nread, NULL);
	} else {
		ReadFile(opwf->winfile, &data.Subchunk2Size, 4, &nread, NULL);
		data.Subchunk2ID = sig;
	}

	// Check it
	if (data.Subchunk2ID != 0x61746164) {
		sprintf(pcmwav_error, "Error in subchunk 2: this is not a PCM WAV file.\n");
		return 0;
	}

	opwf->ndatabytes = data.Subchunk2Size;
	opwf->samplerate = fmt.SampleRate;
	opwf->nchannels = fmt.NumChannels;

	opwf->datapos = SetFilePointer(opwf->winfile, 0, NULL, FILE_CURRENT);
	
	return 1;
}

int pcmwav_read(pcmwavfile *pwf, void *buf, unsigned long len) {

	DWORD	nread;

	ReadFile(pwf->winfile, buf, len, &nread, NULL);

	if (nread != len) {
		sprintf(pcmwav_error, "Error in pcmwav_read(); only read %lu instead of %lu bytes.",
			nread, len);
		return 0;
	}
	
	return 1;
}

int pcmwav_write(pcmwavfile *pwf, void *buf, unsigned long len) {

	DWORD	nwritten;

	WriteFile(pwf->winfile, buf, len, &nwritten, NULL);

	if (nwritten != len) {
		sprintf(pcmwav_error, "Error in pcmwav_write(); only wrote %lu instead of %lu bytes.",
			nwritten, len);
		return 0;
	}
	
	return 1;
}

int pcmwav_rewind(pcmwavfile *pwf) {
	if (!SetFilePointer(pwf->winfile, pwf->datapos, NULL, FILE_BEGIN)) {
		sprintf(pcmwav_error, "Error in pcmwav_rewind().");
		return 0;
	}

	return 1;
}

int pcmwav_seek(pcmwavfile *pwf, long pos) {
	if (!SetFilePointer(pwf->winfile, pos, NULL, FILE_CURRENT)) {
		sprintf(pcmwav_error, "Error in pcmwav_seek() - pos = %ld", pos);
		return 0;
	}

	return 1;
}

int pcmwav_close(pcmwavfile *pwf) {
	CloseHandle(pwf->winfile);
	return 1;
}
